# OssImport

## About this tool
OssImport is a tool to migrate local data or other cloud storage data to OSS. Here're the major features:
- Most common data sources, including local data, Qiniu, Baidu BOS, AWS S3, Azure Blob, Upyun, Tecent COS, Kingsoft KS3, HTTP, OSS, etc.
- Support checkpoint to resume from a failed data migration.
- Traffic throughput throttling.
- Flexible data filtering, based on modified time, file prefix, etc.
- Parallel upload and download. Configurable parallelism.
- Scalable. It supports both single machine mode and distributed mode.

## Runtime Environment
- Java 1.7 or higher

## Deployment

OssImport has two deployment modes: **Standalone** and **Distributed**. For small size data (< 30TB), Standalone is good enough. While for large data migration, distribured mode is recommended.

## Basic Concept
- Job: The whole data migration work. In distributed mode, a job runs on all machines. A job has its own configuration file `local_job.cfg`.
- Task: A sub-job of the data migration job. Depends on data size and file counts, one task could have multiple sub tasks. Each task is responsible for a part of the files.

## Directory structure
Distributed mode:
```
ossimport
├── bin
│   ├── console.jar     # Console model jar
│   ├── master.jar      # Master model jar
│   ├── tracker.jar     # Tracker mode jar
│   └── worker.jar      # Worker mode jar
├── conf
│   ├── job.cfg         # Job configuration file
│   ├── sys.properties  # System run-time properties
│   └── workers         # Worker list
├── console.sh          # The tool's entry point cmd tool, only Linux is supported.
├── logs                # Logs
└── README.md           # Read me file.Please read it before start.
```

**Notes:**
- Three configuration files that are need to change accordingly: `sys.properties`, `job.cfg`, `workers`.
- Read 'README.md' before start.
- 'console.sh' is Linux only.
- Make sure run the tool under the current folder of `console.sh`. That is you should type *bash import.sh* to run the tool.

## Run the data migration service

### Config update
- `conf/job.cfg`: Job configuration. Make sure to specify the actual data source and target parameters before the start.
- `conf/sys.properties`: Sysem run-time properties, such as working directory, worker parameters etc.
- `conf/workers`: worker list.

### Run the cmd

The steps in distributed mode typically are *Sets the config*, *deploy the service*, *clear the previous same name job*, *submit job*, *start the ervice*, *retry failed tasks*, *stop tasks*. They're explained below:
- Deploy the service.In Linux terminal, run "bash console.sh deploy". **Make sure conf/job.cfg, conf/workers are updated correctly before this step**.
- Clear the previous same name job.If the same name job has been run, and you need to start a new job, then the previous job needs to be cleared. If you just want to resume the job or the new job name is different, then skip this step. To clear the previous job, run `bash console.sh clean job_name`.
- Submit job. OssImport cannot re-submit a job with the same name. If there's a previous same name job, please clear it first. The job config file is under `conf/job.cfg`. To submit a job, run `bash console.sh submit [job_cfg_file]`. The job_cfg_file is optional parameter. If it's not specfied, `$OSS_IMPORT_HOME/conf/job.cfg` is used. `$OSS_IMPORT_HOME` by default is `console.sh`'s folder.
- Start the service. Run `bash console.sh start`.
- Retry failed tasks. Tasks could fail due to network or other reasons. To retry these **failed tasks**, just run `bash console.sh retry`. Successful tasks would **not** re-run during the retry.
- Stop tasks.Run  `bash console.sh stop`.

**Tips**: 
- `bash console.sh` would prompt cmd usage when the parameter is not correct;
- Recommend to use full path for specifying the path in the config file and job submit;
- After the job submit, the changes in *job.cfg* will not take effect. Please make sure the correct job.cfg before submitting the job.

## Common failure reasons
- The source file is updated during its upload, In this case, you may see `SIZE_NOT_MATCH` in `log/audit.log`. When it happens, old file has been uploaded, but the updated one is not.
- The source file is deleted during its upload.
- The source file's name does not follow the OSS naming rules (such as cannot start with '/' and not empty).
- Download source file failed due to permission issue or network issue(e.g timeout).
- Application crashes. When it happens, the task status is *Abort*. Please contact us. Thanks.

## Suggestions
For data migration from other cloud, and if you don't have high speed bandwith to access that data,we recommend to use ECS to run the tool.
You could onboard ECS from https://ecs-buy.aliyun.com/#/postpay. And we have following suggetions about how to use ECS:
- Pay with on-demand option.
- Use ECS machines with the same region of the target OSS.
- 100M bandwidth (or more).
- Both Linux and Windows are ok.
- For other configuration (such as CPU, memory, disk, etc), basic model is ok.
- Use internal domain name for the **"targetDomain"** parameter. If the source data comes from OSS of the same region, then **"srcDOmain"** should be internal domain name as well. Using internal domain is not only faster, but also cheaper---it could save the cost of data transfer fee in the download.